indexing
	description:

		"Proxy for Erl-G interpreters"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_INTERPRETER_PROXY

inherit

	AUT_SHARED_RANDOM
		export {NONE} all end

	ERL_G_TYPE_ROUTINES

	EPX_CURRENT_PROCESS
		rename
			random as random_process
		export
			{NONE} all
		end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

	KL_SHARED_OPERATING_SYSTEM
		export {NONE} all end

	EXCEP_CONST
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (an_executable_file_name: STRING;
			a_universe: like universe;
			an_interpreter_log_filename: STRING;
			a_proxy_log_filename: STRING) is
			-- Create a new proxy for the interpreter found at `an_executable_file_name'.
		require
			an_executable_file_name_not_void: an_executable_file_name /= Void
			a_universe_not_void: a_universe /= Void
			an_interpreter_log_filename_not_void: an_interpreter_log_filename /= Void
			a_proxy_log_filename_not_void: a_proxy_log_filename /= Void
		local
			tester: AUT_VARIABLE_EQUALITY_TESTER
		do
			executable_file_name := an_executable_file_name
			melt_path := file_system.dirname (executable_file_name)
			universe := a_universe
			interpreter_log_filename := an_interpreter_log_filename
			create last_instruction.make (1024)
			create response_text.make (1024)
			create last_multi_line_value.make (1024)
			create proxy_log_file.make (a_proxy_log_filename)
			proxy_log_file.open_append
			log_line ("-- A new proxy has been created.")
			create tester.make
			create variable_type_table.make_default
			variable_type_table.set_key_equality_tester (tester)
			create {AUT_NONE_INTERPRETER_RESPONSE_HANDLER} response_handler.make (universe)
		ensure
			executable_file_name_set: executable_file_name = an_executable_file_name
			universe_set: universe = a_universe
			interpreter_log_filename_set: interpreter_log_filename = an_interpreter_log_filename
			proxy_log_file_created: proxy_log_file /= Void
		end

feature -- Status

	is_running: BOOLEAN is
			-- Is the client currently running?
		do
			Result := process /= Void and then not process.is_terminated
		end

	is_ready: BOOLEAN
			-- Is client ready for new commands?

feature -- Access

	last_instruction: STRING
			-- Last instruction sent to interpreter

	response_text: STRING
			-- Textual response of interpreter

	response_handler: AUT_INTERPRETER_RESPONSE_HANDLER
			-- Hander that gets notified of interpreter responses

	is_variable_defined (a_variable: ITP_VARIABLE): BOOLEAN is
			-- Is variable `a_variable' defined in interpreter?
		require
			a_variable_not_void: a_variable /= Void
		do
			Result := variable_type_table.has (a_variable)
		end

	variable_type (a_variable: ITP_VARIABLE): ET_BASE_TYPE is
			-- Type of `a_variable'
		require
			a_variable_not_void: a_variable /= Void
			variable_defined: is_variable_defined (a_variable)
		do
			Result := variable_type_table.item (a_variable)
		ensure
			type_not_void: Result /= Void
		end

	new_variable: ITP_VARIABLE is
			-- Variable not yet defined
		do
			name_generator.generate_new_name
			create Result.make (name_generator.output_string.twin)
			name_generator.output_string.wipe_out
		ensure
			variable_not_void: Result /= Void
			variable_not_defined: not is_variable_defined (Result)
		end

	random_variable: ITP_VARIABLE is
			-- Random variable from interpreter or `Void' if none
			-- defined
		local
			i: INTEGER
			j: INTEGER
			cs: DS_HASH_TABLE_CURSOR [ET_BASE_TYPE, ITP_VARIABLE]
		do
			if variable_type_table.count > 0 then
				random.forth
				i := (random.item  \\ variable_type_table.count) + 1
				from
					j := 1
					cs := variable_type_table.new_cursor
					cs.start
				until
					i = j
				loop
					cs.forth
					j := j + 1
				end
				Result := cs.key
				cs.go_after
			end
		ensure
			variable_defined: Result /= Void implies is_variable_defined (Result)
		end

	random_conforming_variable (a_type: ET_BASE_TYPE): ITP_VARIABLE is
			-- Random variable of `conforming_variables (a_type)' or Void if list
			-- is emtpy
		require
			a_type_not_void: a_type /= Void
		local
			list: like conforming_variables
			cs: DS_LINEAR_CURSOR [ITP_VARIABLE]
			i: INTEGER
			j: INTEGER
		do
			list := conforming_variables (a_type)
			if list.count > 0 then
				random.forth
				i := (random.item  \\ list.count) + 1
				from
					j := 1
					cs := list.new_cursor
					cs.start
				until
					i = j
				loop
					cs.forth
					j := j + 1
				end
				Result := cs.item
				cs.go_after
			end
		ensure
			variable_defined: Result /= Void implies is_variable_defined (Result)
		end

	conforming_variables (a_type: ET_BASE_TYPE): DS_LIST [ITP_VARIABLE] is
			-- All defeined variables conforming to `a_type'
		require
			a_type_not_void: a_type /= Void
		local
			cs: DS_HASH_TABLE_CURSOR [ET_BASE_TYPE, ITP_VARIABLE]
		do
			create {DS_ARRAYED_LIST [ITP_VARIABLE]} Result.make (variable_type_table.count)
			from
				cs := variable_type_table.new_cursor
				cs.start
			until
				cs.off
			loop
				if cs.item.reference_context_conforms_to_type (a_type, a_type, universe) then
					Result.force_last (cs.key)
				end
				cs.forth
			end
		ensure
			variables_not_void: Result /= Void
			variables_doesnt_have_void: not Result.has (Void)
		end

	universe: ET_UNIVERSE
			-- Universe of the interpreter

feature -- Execution

	set_response_handler (a_response_handler: like response_handler) is
			-- Set `a_response_handler' to `response_handler'.
		require
			a_response_handler_not_void: a_response_handler /= Void
		do
			response_handler := a_response_handler
		ensure
			response_handler_set: response_handler = a_response_handler
		end

	start is
			-- Start the client.
		require
			not_running: not is_running
		local
			quoted_log_filename: STRING
		do
			variable_type_table.wipe_out
			if operating_system.is_windows then
				create quoted_log_filename.make (interpreter_log_filename.count + 2)
				quoted_log_filename.append_character ('"')
				quoted_log_filename.append_string (interpreter_log_filename)
				quoted_log_filename.append_character ('"')
			else
				quoted_log_filename := interpreter_log_filename
			end
			create process.make (executable_file_name, <<quoted_log_filename>>)
			process.set_capture_input (True)
			process.set_capture_output (True)
			process.execute
			if is_running then
				create name_generator.make_with_string_stream ("v_")
				create reader.make (process.fd_stdout)
				create expression_printer.make_string (last_instruction)
				last_instruction.wipe_out
				log_line ("-- Proxy has started and connected to interpreter.")
				is_ready := False
				try_read_header
			else
				process := Void
				log_line ("-- Could not start and connect to interpreter.")
			end
		end

	stop is
			-- Close connection to client and terminate it.
			-- If the client is not responsive to a regular shutdown,
			-- its process will be forced to shut down
		require
			is_running: is_running
		local
			failed: BOOLEAN
			posix_process: POSIX_PROCESS
		do
			if not failed then
				print_to_process (":quit%N")
				flush_process
				if process.fd_stdin.is_open_write then
					process.fd_stdin.flush
				end
				millisleep (50)
				if process.fd_stdin.is_owner then
					process.fd_stdin.close
				end
				process.terminate
			end
			if not process.is_terminated then
				posix_process ?= process
				if posix_process /= Void then
						-- Only kill on platforms that support it.
					posix_process.kill (9)
					log_line ("-- Proxy sent kill signal to interpreter.")
				end
			end
			process := Void
			log_line ("-- Proxy has terminated interpreter.")
		rescue
			failed := True
			retry
		end

	create_object (a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE; a_procedure: ET_PROCEDURE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Create new object of type `a_type' using creation
			-- procedure `a_feature' and the arguments `an_argument_list'.
			-- Store the created object in variable `a_receiver'.
		require
			is_running: is_running
			is_ready: is_ready
			a_receiver_not_void: a_receiver /= Void
			a_type_not_void: a_type /= Void
			a_type_not_expanded: not a_type.is_expanded
			a_procedure_not_void: a_procedure /= Void
			a_procedure_is_creation: a_type.direct_base_class (universe).is_creation_exported_to (a_procedure.name, universe.any_class, universe)
			a_procedure_is_not_infix_or_prefix: not a_procedure.name.is_prefix and not a_procedure.name.is_infix
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		local
			response: AUT_INTERPRETER_RESPONSE
		do
			response_has_syntax_error := False
			response_text.wipe_out
			last_instruction.wipe_out
			print_to_process ("create {")
			print_to_process (a_type.to_text)
			print_to_process ("} ")
			print_to_process (a_receiver.name)
			print_to_process (".")
			print_to_process (a_procedure.name.name)
			print_argument_list_to_process (an_argument_list)
			print_to_process ("%N")
			flush_process
			try_parse_multi_line_value
			if response_has_syntax_error then
				create response.make_bad (last_instruction.twin, response_text.twin)
			else
				try_parse_status
				if response_has_syntax_error then
					create response.make_bad (last_instruction.twin, response_text.twin)
				else
					if last_status_success then
						variable_type_table.force (a_type, a_receiver)
						create response.make_success (last_instruction.twin, response_text.twin)
					else
						try_parse_exception
						if response_has_syntax_error then
							create response.make_bad (last_instruction.twin, response_text.twin)
						else
							create response.make_exception (last_instruction.twin, response_text.twin,
																last_exception_code, last_exception_recipient_name,
																last_exception_class_name, last_exception_tag_name,
																last_exception_trace)
						end
					end
					if not response_has_syntax_error then
						try_parse_done
						if response_has_syntax_error then
							create response.make_bad (last_instruction.twin, response_text.twin)
						else
							is_ready := True
						end
					end
				end
			end
			stop_process_on_problems (response)
			response_handler.report_object_creation_response (response, a_receiver, a_type, a_procedure, an_argument_list)
		end

	create_object_default (a_receiver: ITP_VARIABLE; a_type: ET_BASE_TYPE) is
			-- Create new object of type `a_type' using the default creation
			-- procedure. Store the created object in variable `a_receiver'.
		require
			is_running: is_running
			is_ready: is_ready
			a_receiver_not_void: a_receiver /= Void
			a_type_not_void: a_type /= Void
			a_type_not_expanded: not a_type.is_expanded
			is_default_creatable: is_default_creatable (a_type.direct_base_class (universe), universe)
		local
			response: AUT_INTERPRETER_RESPONSE
		do
			response_has_syntax_error := False
			response_text.wipe_out
			last_instruction.wipe_out
			print_to_process ("create {")
			print_to_process (a_type.to_text)
			print_to_process ("} ")
			print_to_process (a_receiver.name)
			print_to_process ("%N")
			flush_process
			try_parse_multi_line_value
			if response_has_syntax_error then
				create response.make_bad (last_instruction.twin, response_text.twin)
			else
				try_parse_status
				if response_has_syntax_error then
					create response.make_bad (last_instruction.twin, response_text.twin)
				else
					if last_status_success then
						variable_type_table.force (a_type, a_receiver)
						create response.make_success (last_instruction.twin, response_text.twin)
					else
						try_parse_exception
						if response_has_syntax_error then
							create response.make_bad (last_instruction.twin, response_text.twin)
						else
							create response.make_exception (last_instruction.twin, response_text.twin,
																last_exception_code, last_exception_recipient_name,
																last_exception_class_name, last_exception_tag_name,
																last_exception_trace)
						end
					end
					if not response_has_syntax_error then
						try_parse_done
						if response_has_syntax_error then
							create response.make_bad (last_instruction.twin, response_text.twin)
						else
							is_ready := True
						end
					end
				end
			end
			stop_process_on_problems (response)
			response_handler.report_default_object_creation_response (response, a_receiver, a_type)
		end

	invoke_feature (a_feature: ET_FEATURE; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Invoke feature `a_feature' from `a_type' with arguments `an_argument_list'.
		require
			is_running: is_running
			is_ready: is_ready
			a_feature_not_void: a_feature /= Void
			a_feature_is_not_infix_or_prefix: not a_feature.name.is_prefix and not a_feature.name.is_infix
			a_target_not_void: a_target /= Void
			a_target_defined: is_variable_defined (a_target)
			a_target_type_not_void: a_target_type /= Void
			a_target_type_not_expanded: not variable_type (a_target).is_expanded
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		local
			response: AUT_INTERPRETER_RESPONSE
		do
			response_has_syntax_error := False
			response_text.wipe_out
			last_instruction.wipe_out
			if variable_type (a_target).direct_base_class (universe).name.name.is_equal (universe.none_class.name.name) then
				create response.make_void_target
			else
				print_to_process (a_target.name)
				print_to_process (".")
				print_to_process (a_feature.name.name)
				print_argument_list_to_process (an_argument_list)
				print_to_process ("%N")
				flush_process
				try_parse_multi_line_value
				if response_has_syntax_error then
					create response.make_bad (last_instruction.twin, response_text.twin)
				else
					try_parse_status
					if response_has_syntax_error then
						create response.make_bad (last_instruction.twin, response_text.twin)
					else
						if last_status_success then
							create response.make_success (last_instruction.twin, response_text.twin)
						else
							try_parse_exception
							if response_has_syntax_error then
								create response.make_bad (last_instruction.twin, response_text.twin)
							else
								create response.make_exception (last_instruction.twin, response_text.twin,
																	last_exception_code, last_exception_recipient_name,
																	last_exception_class_name, last_exception_tag_name,
																	last_exception_trace)
							end
						end
						if not response_has_syntax_error then
							try_parse_done
							if response_has_syntax_error then
								create response.make_bad (last_instruction.twin, response_text.twin)
							else
								is_ready := True
							end
						end
					end
				end
				stop_process_on_problems (response)
			end
			response_handler.report_feature_invocation_response (response, a_feature, a_target, a_target_type, an_argument_list)
		end

	invoke_and_assign_feature (a_receiver: ITP_VARIABLE; a_query: ET_QUERY; a_target: ITP_VARIABLE; a_target_type: ET_BASE_TYPE; an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Invoke query `a_query' from `a_type' with arguments `an_argument_list'.
			-- Store result in variable `a_receiver'.
		require
			is_running: is_running
			is_ready: is_ready
			a_receiver_not_void: a_receiver /= Void
			a_query_not_void: a_query /= Void
			a_query_is_not_infix_or_prefix: not a_query.name.is_prefix and not a_query.name.is_infix
			a_target_not_void: a_target /= Void
			a_target_defined: is_variable_defined (a_target)
			a_target_type_not_void: a_target_type /= Void
			a_target_type_not_expanded: not variable_type (a_target).is_expanded
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		local
			response: AUT_INTERPRETER_RESPONSE
			instruction: STRING
		do
			response_has_syntax_error := False
			response_text.wipe_out
			last_instruction.wipe_out
			if variable_type (a_target).direct_base_class (universe).name.name.is_equal (universe.none_class.name.name) then
				create response.make_void_target
			else
				print_to_process (a_receiver.name)
				print_to_process (" := ")
				print_to_process (a_target.name)
				print_to_process (".")
				print_to_process (a_query.name.name)
				print_argument_list_to_process (an_argument_list)
				print_to_process ("%N")
				flush_process
				try_parse_multi_line_value
				if response_has_syntax_error then
					create response.make_bad (last_instruction.twin, response_text.twin)
				else
					try_parse_status
					if response_has_syntax_error then
						create response.make_bad (last_instruction.twin, response_text.twin)
					else
						if last_status_success then
							try_parse_done
							if response_has_syntax_error then
								create response.make_bad (last_instruction.twin, response_text.twin)
							else
								is_ready := True
								instruction := last_instruction.twin
								retrieve_type_of_variable (a_receiver)
								if response_has_syntax_error then
									create response.make_bad (instruction, response_text.twin)
								else
									variable_type_table.force (last_type, a_receiver)
									create response.make_success (instruction, response_text.twin)
									is_ready := True
								end
							end
						else
							try_parse_exception
							if response_has_syntax_error then
								create response.make_bad (last_instruction.twin, response_text.twin)
							else
								create response.make_exception (last_instruction.twin, response_text.twin,
																	last_exception_code, last_exception_recipient_name,
																	last_exception_class_name, last_exception_tag_name,
																	last_exception_trace)
							end
							if not response_has_syntax_error then
								try_parse_done
								if response_has_syntax_error then
									create response.make_bad (last_instruction.twin, response_text.twin)
								else
									is_ready := True
								end
							end
						end
					end
				end
				stop_process_on_problems (response)
			end
			response_handler.report_feature_invocation_and_assignment_response (response, a_receiver, a_query, a_target, a_target_type, an_argument_list)
		end

	assign_expression (a_receiver: ITP_VARIABLE; an_expression: ITP_EXPRESSION) is
			-- Assign `a_constant' to `a_receiver'.
		require
			is_running: is_running
			is_ready: is_ready
			a_receiver_not_void: a_receiver /= Void
			a_constant_not_void: an_expression /= Void
		local
			response: AUT_INTERPRETER_RESPONSE
			instruction: STRING
		do
			response_has_syntax_error := False
			response_text.wipe_out
			last_instruction.wipe_out
			print_to_process (a_receiver.name)
			print_to_process (" := ")
			an_expression.process (expression_printer)
			print_to_process ("%N")
			flush_process
			try_parse_done
			if response_has_syntax_error then
				create response.make_bad (last_instruction.twin, response_text.twin)
			else
				is_ready := True
				instruction := last_instruction.twin
				retrieve_type_of_variable (a_receiver)
				if response_has_syntax_error then
					create response.make_bad (instruction, response_text.twin)
				else
					create response.make_success (instruction, response_text.twin)
					variable_type_table.force (last_type, a_receiver)
				end
			end
			response_handler.report_expression_assignment_response (response, a_receiver, an_expression)
		end

feature {NONE} -- Implementation

	stop_process_on_problems (a_response: AUT_INTERPRETER_RESPONSE) is
			-- Stop `process' if a class invariant has occured in the interpreter or
			-- a bad response has been received.
			-- The interpreter will shut down in case of a class invariant, because the
			-- system will be in an invalid state.
		require
			is_running: is_running
			a_response_not_void: a_response /= Void
		do
			if
				a_response.was_exception_thrown and then
				a_response.exception_code = Class_invariant
			then
				stop
				log_line ("-- Proxy has terminated interpreter due to class invariant violation.")
			elseif
				a_response.was_bad
			then
				stop
				log_line ("-- Proxy has terminated interpreter due to bad behavior.")
			end
		end

	retrieve_type_of_variable (a_variable: ITP_VARIABLE) is
			-- Retrieve the type of variable `a_variable' and
			-- make it available via `last_type'
		require
			a_variable_not_void: a_variable /= Void
			is_running: is_running
			is_ready: is_ready
		do
			last_instruction.wipe_out
			print_to_process (":type ")
			print_to_process (a_variable.name)
			print_to_process ("%N")
			flush_process
			try_parse_type_name
			try_parse_done
			if not response_has_syntax_error then
				is_ready := True
			end
		ensure
			result_or_error: response_has_syntax_error or last_type /= Void
		end

	try_parse_type_name is
			-- Parse type name.
		require
			is_running: is_running
			not_ready: not is_ready
		do
			try_read_line
			if reader.has_read_line then
				last_type := class_type (reader.last_line, universe)
				if last_type = Void then
					set_response_has_syntax_error
				end
			end
		ensure
			result_or_error: response_has_syntax_error or last_type /= Void
		end

	try_parse_status is
			-- Parse "status:" response.
		require
			is_running: is_running
			not_ready: not is_ready
		do
			try_read_line
			if reader.has_read_line then
				if reader.last_line.is_equal ("status: success") then
					last_status_success := True
				elseif reader.last_line.is_equal ("status: exception") then
					last_status_success := False
				else
					set_response_has_syntax_error
				end
			else
				set_response_has_syntax_error
			end
		end

	try_parse_done is
			-- Parse "done:" response.
		require
			is_running: is_running
			not_ready: not is_ready
		do
			try_read_line
			if reader.has_read_line then
				if not reader.last_line.is_equal ("done:") then
					set_response_has_syntax_error
				end
			else
				set_response_has_syntax_error
			end
		end

	try_parse_multi_line_value is
			-- Parse the response from client, but abort if its taking
			-- too long.
		require
			is_running: is_running
			not_ready: not is_ready
		do
			try_read_line
			if reader.has_read_line then
				if reader.last_line.is_equal (multi_line_value_start_tag) then
					from
						try_read_line
						last_multi_line_value.wipe_out
					until
						not reader.has_read_line or else
						(reader.last_line.count >= multi_line_value_end_tag.count and then
						 reader.last_line.substring (reader.last_line.count - multi_line_value_end_tag.count + 1,
															  reader.last_line.count).is_equal (multi_line_value_end_tag))
					loop
						last_multi_line_value.append_string (reader.last_line)
						last_multi_line_value.append_character ('%N')
						try_read_line
					end
					if not reader.has_read_line then
						set_response_has_syntax_error
					end
				else
					set_response_has_syntax_error
				end
			else
				set_response_has_syntax_error
			end
		end

	try_parse_exception is
			-- Parse an exception.
		require
			is_running: is_running
			not_ready: not is_ready
		do
			try_read_line
			if reader.has_read_line and then reader.last_line.is_integer then
				last_exception_code := reader.last_line.to_integer
				try_read_line
				if reader.has_read_line then
					last_exception_recipient_name := reader.last_line
					try_read_line
					if reader.has_read_line then
						last_exception_class_name := reader.last_line
						try_read_line
						if reader.has_read_line then
							last_exception_tag_name := reader.last_line
							try_parse_multi_line_value
							if reader.has_read_line then
								last_exception_trace := last_multi_line_value
							else
								set_response_has_syntax_error
							end
						else
							set_response_has_syntax_error
						end
					else
						set_response_has_syntax_error
					end
				else
					set_response_has_syntax_error
				end

			else
				set_response_has_syntax_error
			end
		end

	set_response_has_syntax_error is
			-- Set `repsonse_has_syntax_error' to `True'.
		require
			response_has_no_syntax_error_set: not response_has_syntax_error
		do
			response_has_syntax_error := True
			log_line ("-- Proxy received bad response.")
		ensure
			response_has_syntax_error_set: response_has_syntax_error
		end


	process: EPX_EXEC_PROCESS
			-- Client process

	reader: AUT_NON_BLOCKING_LINE_READER
			-- Non blocking reader for client

	timeout: INTEGER is 5
			-- Client timeout in seconds

	last_multi_line_value: STRING
			-- Last multi line value parsed (without the start and end tag)

	last_exception_code: INTEGER
			-- Code of last exception

	last_exception_recipient_name: STRING
			-- Name of last exception's recipient

	last_exception_class_name: STRING
			-- Name of last exception's class name

	last_exception_tag_name: STRING
			-- Name of last exception's tag name

	last_exception_trace: STRING
			-- Stack trace of last exception

	print_argument_list_to_process (an_argument_list: DS_LINEAR [ITP_EXPRESSION]) is
			-- Print argument list `an_arinstruction' to `process.fd_stdin'.
		require
			an_argument_list_not_void: an_argument_list /= Void
			an_argument_list_doesnt_have_void: not an_argument_list.has (Void)
		local
			cs: DS_LINEAR_CURSOR [ITP_EXPRESSION]
		do
			if an_argument_list.count > 0 then
				print_to_process (" (")
				from
					cs := an_argument_list.new_cursor
					cs.start
				until
					cs.off
				loop
					cs.item.process (expression_printer)
					cs.forth
					if not cs.after then
						print_to_process (", ")
					end
				end
				print_to_process (")")
			end
		end

	print_to_process (a_string: STRING) is
			-- Print `a_string' to `process.fd_stdin'. Recover from the
			-- case that printing was not possible due to a broken pipe.
		require
			a_string_not_void: a_string /= Void
		do
			last_instruction.append_string (a_string)
		end

	flush_process is
			-- Send content of `prcoess_buffer' to process stdin.
		local
			failed: BOOLEAN
		do
			if not failed then
				is_ready := False
				if process.fd_stdin.is_open_write then
					process.fd_stdin.put_string (last_instruction)
					log (last_instruction)
				else
					log ("Error: could not send instruction to interpreter due its input stream being closed.")
				end
			end
		rescue
			failed := True
			retry
		end

feature {NONE} -- Implementation

	last_type: ET_BASE_TYPE
			-- Last type queried from interpreter

	last_status_success: BOOLEAN
			-- Was last status responsed parsed 'success'?

	executable_file_name: STRING
			-- File name of interpreter executable

	interpreter_log_filename: STRING
			-- File name of the interpreters log

	melt_path: STRING
			-- Path where melt file of test client resides

	name_generator: AUT_UNIQUE_NAME_GENERATOR
			-- Name generator for variable names

	expression_printer: AUT_EXPRESSION_PRINTER
			-- Expression printer

	response_has_syntax_error: BOOLEAN
			-- Is there a syntax error in the response from the interpreter

	variable_type_table: DS_HASH_TABLE [ET_BASE_TYPE, ITP_VARIABLE]

	try_read_line is
			-- Try to read a line from the interpreter or time out
		require
			is_running: is_running
			not_ready: not is_ready
		local
			start_time: STDC_TIME
			l_current_time: STDC_TIME
		do
			create start_time.make_from_now
			create l_current_time.make_from_now
			from
				reader.try_read_line
			until
				reader.has_read_line or l_current_time.value > start_time.value + timeout
			loop
				reader.try_read_line
				l_current_time.make_from_now
				millisleep (50)
			end
			if reader.has_read_line then
				log_line (interpreter_log_prefix + reader.last_line)
				response_text.append_string (reader.last_line)
				response_text.append_character ('%N')
			end
		end

	ignore_lines (a_count: INTEGER) is
			-- Read `a_count' lines from process and
			-- ignore content.
		require
			a_count_valid: a_count >= 1
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > a_count or reader.end_of_input
			loop
				reader.try_read_line
				if reader.has_read_line then
					log_line (interpreter_log_prefix + reader.last_line)
					i := i + 1
				end
			end
		end

	try_read_header is
			-- Try to read the header message printed by executables
			-- compiled with the free edition of ISE Eiffel
			-- on Unix.
		do
			if operating_system.is_unix then
				try_read_line
				if reader.has_read_line then
					check
						header_valid: reader.last_line.is_equal ("This program has been produced with a demo or non-commercial version")
					end
					ignore_lines (18)
				end
			end
			is_ready := is_running
		end

feature {NONE} -- Logging

	proxy_log_file: KL_TEXT_OUTPUT_FILE
			-- Proxy log file

	log (a_string: STRING) is
			-- Log `a_string' to `log_file'.
		require
			a_string_not_void: a_string /= Void
		do
			if proxy_log_file.is_open_write then
				proxy_log_file.put_string (a_string)
				proxy_log_file.flush
			end
		end

	log_line (a_string: STRING) is
			-- Log `a_string' followed by a new-line character to `log_file'.
		require
			a_string_not_void: a_string /= Void
		do
			log (a_string)
			log ("%N")
		end

feature {NONE} -- Constants

	multi_line_value_start_tag: STRING is "---multi-line-value-start---"
			-- Multi line start tag

	multi_line_value_end_tag: STRING is "---multi-line-value-end---"
			-- Multi line end tag

	interpreter_log_prefix: STRING is "%T-- > "
			-- Prefix of messages sent from interpreter in log file

invariant

	universe_not_void: universe /= Void
	process_not_void: is_running implies process /= Void
	is_running_implies_reader: is_running implies reader /= Void
	name_generator_not_void: is_running implies name_generator /= Void
	expression_printer_not_void: is_running implies expression_printer /= Void
	executable_file_name_not_void: executable_file_name /= Void
	interpreter_log_file_name_not_void: interpreter_log_filename /= Void
	melt_path_not_void: melt_path /= Void
	not_running_implies_not_ready: not is_running implies not is_ready
	is_ready_implies_is_running: is_ready implies is_running
	proxy_log_file_not_void: proxy_log_file /= Void
	last_instruction_not_void: last_instruction /= Void
	response_text_not_void: response_text /= Void
	variable_type_table_not_void: variable_type_table /= Void
	response_handler_not_void: response_handler /= Void
	last_multi_line_value_not_void: last_multi_line_value /= Void

end
