indexing

	description:

		"Objects that instruct interpreter to call features using random input"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_RANDOM_FEATURE_CALLER

inherit

	AUT_TASK

	AUT_SHARED_RANDOM
		export {NONE} all end

	ERL_G_TYPE_ROUTINES
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_universe: like universe; an_interpreter: like interpreter) is
			-- Create new feature caller.
		require
			a_universe_not_void: a_universe /= Void
			a_interpreter_not_void: an_interpreter /= Void
		do
			universe := a_universe
			interpreter := an_interpreter
		ensure
			universe_set: universe = a_universe
			interpreter_set: interpreter = an_interpreter
		end

feature -- Status

	has_next_step: BOOLEAN is
			-- Is there a next step to execute?
		do
			Result := interpreter.is_running and interpreter.is_ready and not steps_completed
		end

feature -- Access

	feature_: ET_FEATURE
			-- Feature to call; If `Void' a random feature
			-- will be selected.

	type: ET_BASE_TYPE
			-- Type of feature; If `Void' a random type
			-- will be selected.

	target: ITP_VARIABLE
			-- Target of feature call

	receiver: ITP_VARIABLE
			-- Target of call

	input_creator: AUT_RANDOM_INPUT_CREATOR
			-- Input creator used to create arguments for feature call

	feature_caller: AUT_RANDOM_FEATURE_CALLER
			-- Feature call for diversification of object pool

	universe: ET_UNIVERSE
			-- Universe

	interpreter: AUT_INTERPRETER_PROXY
			-- Proxy to the interpreter used to execute call

feature -- Change

	set_feature_and_type (a_feature: like feature_; a_type: like type) is
				-- Set `feature_' to `a_feature' and
				-- `type' to `a_type'.
		require
			a_feature_not_void: a_feature /= Void
			a_type_not_void: a_type /= Void
		do
			feature_ := a_feature
			type := a_type
		ensure
			feature_set: feature_ = a_feature
			type_set: type = a_type
		end

feature -- Execution

	start is
		do
			steps_completed := False
			input_creator := Void
			feature_caller := Void
			target := Void
			receiver := Void
		ensure then
			input_creator_void: input_creator = Void
			feature_caller_void: feature_caller = Void
			target_void: target = Void
			receiver_void: receiver = Void
		end

	step is
		do
			if type = Void then
				-- 1st step in diversify mode
				create_input_creator_diversify
			elseif input_creator = Void then
				-- 1st step in non-diversify mode
				create_input_creator
				create feature_caller.make (universe, interpreter)
				feature_caller.start
			elseif input_creator.has_next_step then
				input_creator.step
			elseif feature_caller /= Void and then feature_caller.has_next_step then
				feature_caller.step
			else
				invoke
				steps_completed := True
			end
		end

	cancel is
		do
			steps_completed := True
		end

feature {NONE} -- Implementation

	steps_completed: BOOLEAN
			-- Should there be no more steps for reasons other than a bad interpeter?

feature {NONE} -- Steps

	create_input_creator_diversify is
			-- Create `input_creator' for use in diversify mode.
		local
			i: INTEGER
			count: INTEGER
		do
			target := interpreter.random_variable
			if target /= Void and then not interpreter.variable_type (target).is_expanded then
				type := interpreter.variable_type (target)
				choose_feature
				if feature_ /= Void then
					create input_creator.make (universe, interpreter)
					if feature_.arguments /= Void then
						from
							i := 1
							count := feature_.arguments.count
						until
							i > count
						loop
							input_creator.add_type (feature_.arguments.item (i).type.base_type (type, universe))
							i := i + 1
						end
					end
					input_creator.start
				else
					type := Void
				end
			else
				steps_completed := True
			end
		ensure
			feature_and_type_set: has_next_step = ((type /= Void) and (feature_ /= Void))
		end

	create_input_creator is
			-- Create `input_creator' for use in non-diversify mode.
		require
			type_not_void: type /= Void
			feature_not_void: feature_ /= Void
		local
			i: INTEGER
			count: INTEGER
		do
			create input_creator.make (universe, interpreter)
			input_creator.add_type (type)
			if feature_.arguments /= Void then
				from
					i := 1
					count := feature_.arguments.count
				until
					i > count
				loop
					input_creator.add_type (feature_.arguments.item (i).type.base_type (type, universe))
					i := i + 1
				end
			end
			input_creator.start
		end

	invoke is
			-- Invoke feature to call.
		require
			type_not_void: type /= Void
			type_not_expanded: not type.is_expanded
			feature_not_void: feature_ /= Void
			input_creator_not_void: input_creator /= Void
			input_creator_done: feature_.arguments /= Void and target /= Void implies input_creator.receivers.count = feature_.arguments.count
			input_creator_done: feature_.arguments /= Void and target = Void implies input_creator.receivers.count = feature_.arguments.count + 1
		local
			list: DS_LIST [ITP_EXPRESSION]
			query: ET_QUERY
		do
			list := input_creator.receivers
			if target = Void then
				target ?= list.first
					-- If `list.first' represents `Void' (ITP_REFERENCE), target will be `Void'.
				list.remove_first
			end
			if target /= Void then
				query ?= feature_
				if query /= Void then
					receiver := interpreter.new_variable
					interpreter.invoke_and_assign_feature (receiver, query, target, type, input_creator.receivers)
				else
					interpreter.invoke_feature (feature_, target, type, input_creator.receivers)
				end
			end
		end

feature {NONE} -- Implementation

	choose_feature is
			-- Chose `feature_' from `type'.
		require
			type_not_void: type /= Void
		local
			count: INTEGER
			class_: ET_CLASS
			i: INTEGER
		do
			class_ := type.direct_base_class (universe)
			random.forth
			count := feature_count (class_, universe)
			if count > 0 then
				i := (random.item  \\ count) + 1
				feature_ := i_th_feature (i, class_, universe)
			else
				steps_completed := True
			end
		ensure
			end_or_feature_not_void: not has_next_step xor (feature_ /= Void)
		end

invariant

	universe_not_void: universe /= Void
	interpreter_not_void: interpreter /= Void
	type_and_feature_valid: (type /= Void) = (feature_ /= Void)

end
