indexing

	description:

		"Objects that instruct interpreter to create an object"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class AUT_RANDOM_OBJECT_CREATOR

inherit

	AUT_TASK

	AUT_SHARED_RANDOM
		export {NONE} all end

	ERL_G_TYPE_ROUTINES
		export {NONE} all end

	KL_SHARED_STRING_EQUALITY_TESTER
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_universe: like universe; an_interpreter: like interpreter; a_type: like type) is
			-- Create new feature caller.
		require
			a_universe_not_void: a_universe /= Void
			a_interpreter_not_void: an_interpreter /= Void
			a_type_not_void: a_type /= Void
		do
			universe := a_universe
			interpreter := an_interpreter
			type := a_type
		ensure
			universe_set: universe = a_universe
			interpreter_set: interpreter = an_interpreter
			type_set: type = a_type
		end

feature -- Status

	has_next_step: BOOLEAN is
			-- Is there a next step to execute?
		do
			Result := interpreter.is_running and interpreter.is_ready and not steps_completed
		end

feature -- Access

	creation_procedure: ET_PROCEDURE
			-- Creation procedure used to create object

	use_default_creation_procedure: BOOLEAN
			-- Should the default creation procedure be used

	type: ET_BASE_TYPE
			-- Type of object to create

	receiver: ITP_EXPRESSION
			-- Variable that will reference created object in interpreter

	input_creator: AUT_RANDOM_INPUT_CREATOR
			-- Input creator used to create arguments for creation procedure call

	universe: ET_UNIVERSE
			-- Universe

	interpreter: AUT_INTERPRETER_PROXY
			-- Proxy to the interpreter used to execute call

feature -- Change

	set_creation_procedure (a_creation_procedure: like creation_procedure) is
				-- Set `creation_procedure' to `a_creation_procedure'.
		require
			a_creation_procedure_not_void: a_creation_procedure /= Void
		do
			creation_procedure := a_creation_procedure
		ensure
			creation_procedure_set: creation_procedure = a_creation_procedure
		end

feature -- Execution

	start is
		do
			steps_completed := False
			input_creator := Void
			receiver := Void
		ensure then
			input_creator_void: input_creator = Void
			receiver_void: receiver = Void
		end

	step is
		local
			variable: ITP_VARIABLE
		do
			if type.is_expanded then
				choose_expanded_receiver
				steps_completed := True
			elseif creation_procedure = Void and not use_default_creation_procedure then
				choose_creation_procedure
			elseif input_creator = Void then
				create_input_creator
			elseif input_creator.has_next_step then
				input_creator.step
			elseif receiver = Void then
				receiver := interpreter.new_variable
			else
				variable ?= receiver
				check
					variable_not_void: variable /= Void
				end
				if use_default_creation_procedure then
					interpreter.create_object_default (variable, type)
				else
					interpreter.create_object (variable, type, creation_procedure,
												input_creator.receivers)
				end
				if not interpreter.is_variable_defined (variable) then
					create {ITP_REFERENCE} receiver.make (Void)
				end
				steps_completed := True
			end
		end

	cancel is
		do
			steps_completed := True
		end

feature {NONE} -- Implementation

	steps_completed: BOOLEAN
			-- Should there be no more steps for reasons other than a bad interpeter?

feature {NONE} -- Steps

	create_input_creator is
			-- Create `input_creator'.
		require
			has_creation_procedure: creation_procedure /= Void or use_default_creation_procedure
		local
			i: INTEGER
			count: INTEGER
		do
			create input_creator.make (universe, interpreter)
			if creation_procedure /= Void and then creation_procedure.arguments /= Void then
				from
					i := 1
					count := creation_procedure.arguments.count
				until
					i > count
				loop
					input_creator.add_type (creation_procedure.arguments.item (i).type.base_type (type, universe))
					i := i + 1
				end
			end
			input_creator.start
		end

	choose_creation_procedure is
			-- Choose a creation procedure (including default creation procedure)
			-- of `type' and set `creation_procedure' to it. Note that
			-- instead of choosing a creation procedure
			-- also `Void' can be choosen.
		require
			type_not_expanded: not type.is_expanded
		local
			count: INTEGER
			i: INTEGER
			j: INTEGER
			void_constant: ITP_REFERENCE
			class_: ET_CLASS
			creator_index: INTEGER
			feature_index: INTEGER
			creator: ET_CREATOR
			variable: ITP_VARIABLE
		do
			class_ := type.direct_base_class (universe)
			random.forth
			count := creation_procedure_count (type, class_, universe)
			i := random.item  \\ (count + 1)
			if i = 0 then
				-- Choose Void
				variable := interpreter.new_variable
				receiver := variable
				create void_constant.make (Void)
				interpreter.assign_expression (variable, void_constant)
				steps_completed := True
			elseif i = count and is_default_creatable (class_, universe) then
				use_default_creation_procedure := True
			else
				check
					has_creators: class_.creators /= Void
				end
				from
					creator_index := 1
					j := 1
				until
					creator_index > class_.creators.count or creation_procedure /= Void
				loop
					creator := class_.creators.item (creator_index)
					if creator.clients.has_class (universe.any_class) then
						from
							feature_index := 1
						until
							feature_index > creator.count or creation_procedure /= Void
						loop
							if j = i then
								creation_procedure := class_.named_procedure (creator.feature_name (feature_index))
							end
							feature_index := feature_index + 1
							j := j + 1
						end
					end
					creator_index := creator_index + 1
				end
				check
					has_creation_procedure: creation_procedure /= Void
				end
			end
		end

	choose_expanded_receiver is
			-- Choose an expanded receiver and set it to
			-- `receiver'.
		require
			type.is_expanded
			is_basic_type: True -- TODO: correct assertion
		local
			class_: ET_CLASS
		do
			class_ := type.direct_base_class (universe)
			if string_equality_tester.test (class_.name.name, "BOOLEAN") then
				choose_boolean_receiver
			elseif string_equality_tester.test (class_.name.name, "CHARACTER") then
				choose_character_receiver
			elseif string_equality_tester.test (class_.name.name, "DOUBLE") then
				choose_double_receiver
			elseif string_equality_tester.test (class_.name.name, "INTEGER_8") then
				choose_integer_8_receiver
			elseif string_equality_tester.test (class_.name.name, "INTEGER_16") then
				choose_integer_16_receiver
			elseif string_equality_tester.test (class_.name.name, "INTEGER") then
				choose_integer_receiver
			elseif string_equality_tester.test (class_.name.name, "INTEGER_32") then
				choose_integer_receiver
			elseif string_equality_tester.test (class_.name.name, "INTEGER_64") then
				choose_integer_64_receiver
			elseif string_equality_tester.test (class_.name.name, "NATURAL_8") then
				choose_natural_8_receiver
			elseif string_equality_tester.test (class_.name.name, "NATURAL_16") then
				choose_natural_16_receiver
			elseif string_equality_tester.test (class_.name.name, "NATURAL_32") then
				choose_natural_32_receiver
			elseif string_equality_tester.test (class_.name.name, "NATURAL_64") then
				choose_natural_64_receiver
			elseif string_equality_tester.test (class_.name.name, "POINTER") then
				choose_pointer_receiver
			elseif string_equality_tester.test (class_.name.name, "REAL") then
				choose_real_receiver
			else
				check
					todo: False -- TODO: custom expanded types not yet implemented
				end
			end
		end

	choose_boolean_receiver is
			-- Choose boolean receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 1)
			if i = 0 then
				create {ITP_BOOLEAN} receiver.make (False)
			else
				create {ITP_BOOLEAN} receiver.make (True)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_character_receiver is
			-- Choose character receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 255)
			create {ITP_CHARACTER} receiver.make (i.to_character)
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_double_receiver is
			-- Choose double receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 10)
			inspect i
			when 0 then
				create {ITP_DOUBLE} receiver.make (-1.0)
			when 1 then
				create {ITP_DOUBLE} receiver.make (1.0)
			when 2 then
				create {ITP_DOUBLE} receiver.make (0.0)
			when 3 then
				create {ITP_DOUBLE} receiver.make (-2.0)
			when 4 then
				create {ITP_DOUBLE} receiver.make (2.0)
			when 5 then
				create {ITP_DOUBLE} receiver.make (3.14159265358979323846)
			when 6 then
				create {ITP_DOUBLE} receiver.make (-2.7182818284590452354)
			when 7 then
				create {ITP_DOUBLE} receiver.make (1.7976931348623157e+308)
			when 8 then
				create {ITP_DOUBLE} receiver.make (2.2250738585072014e-308)
			when 9 then
				create {ITP_DOUBLE} receiver.make (2.2204460492503131e-16)
			else
				check
					dead_end: False
				end
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_integer_8_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_INTEGER_8} receiver.make (-1)
			elseif i = 1 then
				create {ITP_INTEGER_8} receiver.make (0)
			elseif i = 2 then
				create {ITP_INTEGER_8} receiver.make (1)
			elseif i = 3 then
				create {ITP_INTEGER_8} receiver.make (2)
			elseif i = 4 then
				create {ITP_INTEGER_8} receiver.make (3)
			elseif i = 5 then
				create {ITP_INTEGER_8} receiver.make (4)
			elseif i = 6 then
				create {ITP_INTEGER_8} receiver.make (5)
			elseif i = 7 then
				create {ITP_INTEGER_8} receiver.make (6)
			elseif i = 8 then
				create {ITP_INTEGER_8} receiver.make (7)
			elseif i = 9 then
				create {ITP_INTEGER_8} receiver.make (8)
			elseif i = 10 then
				create {ITP_INTEGER_8} receiver.make (9)
			elseif i = 11 then
				create {ITP_INTEGER_8} receiver.make (-2)
			elseif i = 12 then
				create {ITP_INTEGER_8} receiver.make (10)
			elseif i = 13 then
				create {ITP_INTEGER_8} receiver.make (-10)
			elseif i = 14 then
				create {ITP_INTEGER_8} receiver.make (100)
			elseif i = 15 then
				create {ITP_INTEGER_8} receiver.make (-100)
			elseif i = 16 then
				create {ITP_INTEGER_8} receiver.make ({INTEGER_8}.Min_value)
			elseif i = 17 then
				create {ITP_INTEGER_8} receiver.make ({INTEGER_8}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_integer_16_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_INTEGER_16} receiver.make (-1)
			elseif i = 1 then
				create {ITP_INTEGER_16} receiver.make (0)
			elseif i = 2 then
				create {ITP_INTEGER_16} receiver.make (1)
			elseif i = 3 then
				create {ITP_INTEGER_16} receiver.make (2)
			elseif i = 4 then
				create {ITP_INTEGER_16} receiver.make (3)
			elseif i = 5 then
				create {ITP_INTEGER_16} receiver.make (4)
			elseif i = 6 then
				create {ITP_INTEGER_16} receiver.make (5)
			elseif i = 7 then
				create {ITP_INTEGER_16} receiver.make (6)
			elseif i = 8 then
				create {ITP_INTEGER_16} receiver.make (7)
			elseif i = 9 then
				create {ITP_INTEGER_16} receiver.make (8)
			elseif i = 10 then
				create {ITP_INTEGER_16} receiver.make (9)
			elseif i = 11 then
				create {ITP_INTEGER_16} receiver.make (-2)
			elseif i = 12 then
				create {ITP_INTEGER_16} receiver.make (10)
			elseif i = 13 then
				create {ITP_INTEGER_16} receiver.make (-10)
			elseif i = 14 then
				create {ITP_INTEGER_16} receiver.make (100)
			elseif i = 15 then
				create {ITP_INTEGER_16} receiver.make (-100)
			elseif i = 16 then
				create {ITP_INTEGER_16} receiver.make ({INTEGER_16}.Min_value)
			elseif i = 17 then
				create {ITP_INTEGER_16} receiver.make ({INTEGER_16}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_integer_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_INTEGER} receiver.make (-1)
			elseif i = 1 then
				create {ITP_INTEGER} receiver.make (0)
			elseif i = 2 then
				create {ITP_INTEGER} receiver.make (1)
			elseif i = 3 then
				create {ITP_INTEGER} receiver.make (2)
			elseif i = 4 then
				create {ITP_INTEGER} receiver.make (3)
			elseif i = 5 then
				create {ITP_INTEGER} receiver.make (4)
			elseif i = 6 then
				create {ITP_INTEGER} receiver.make (5)
			elseif i = 7 then
				create {ITP_INTEGER} receiver.make (6)
			elseif i = 8 then
				create {ITP_INTEGER} receiver.make (7)
			elseif i = 9 then
				create {ITP_INTEGER} receiver.make (8)
			elseif i = 10 then
				create {ITP_INTEGER} receiver.make (9)
			elseif i = 11 then
				create {ITP_INTEGER} receiver.make (-2)
			elseif i = 12 then
				create {ITP_INTEGER} receiver.make (10)
			elseif i = 13 then
				create {ITP_INTEGER} receiver.make (-10)
			elseif i = 14 then
				create {ITP_INTEGER} receiver.make (100)
			elseif i = 15 then
				create {ITP_INTEGER} receiver.make (-100)
			elseif i = 16 then
				create {ITP_INTEGER} receiver.make ({INTEGER}.Min_value)
			elseif i = 17 then
				create {ITP_INTEGER} receiver.make ({INTEGER}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_integer_64_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_INTEGER_64} receiver.make (-1)
			elseif i = 1 then
				create {ITP_INTEGER_64} receiver.make (0)
			elseif i = 2 then
				create {ITP_INTEGER_64} receiver.make (1)
			elseif i = 3 then
				create {ITP_INTEGER_64} receiver.make (2)
			elseif i = 4 then
				create {ITP_INTEGER_64} receiver.make (3)
			elseif i = 5 then
				create {ITP_INTEGER_64} receiver.make (4)
			elseif i = 6 then
				create {ITP_INTEGER_64} receiver.make (5)
			elseif i = 7 then
				create {ITP_INTEGER_64} receiver.make (6)
			elseif i = 8 then
				create {ITP_INTEGER_64} receiver.make (7)
			elseif i = 9 then
				create {ITP_INTEGER_64} receiver.make (8)
			elseif i = 10 then
				create {ITP_INTEGER_64} receiver.make (9)
			elseif i = 11 then
				create {ITP_INTEGER_64} receiver.make (-2)
			elseif i = 12 then
				create {ITP_INTEGER_64} receiver.make (10)
			elseif i = 13 then
				create {ITP_INTEGER_64} receiver.make (-10)
			elseif i = 14 then
				create {ITP_INTEGER_64} receiver.make (100)
			elseif i = 15 then
				create {ITP_INTEGER_64} receiver.make (-100)
			elseif i = 16 then
				create {ITP_INTEGER_64} receiver.make ({INTEGER_64}.Min_value)
			elseif i = 17 then
				create {ITP_INTEGER_64} receiver.make ({INTEGER_64}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_natural_8_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_NATURAL_8} receiver.make (0)
			elseif i = 1 then
				create {ITP_NATURAL_8} receiver.make (1)
			elseif i = 2 then
				create {ITP_NATURAL_8} receiver.make (2)
			elseif i = 3 then
				create {ITP_NATURAL_8} receiver.make (3)
			elseif i = 4 then
				create {ITP_NATURAL_8} receiver.make (4)
			elseif i = 5 then
				create {ITP_NATURAL_8} receiver.make (5)
			elseif i = 6 then
				create {ITP_NATURAL_8} receiver.make (6)
			elseif i = 7 then
				create {ITP_NATURAL_8} receiver.make (7)
			elseif i = 8 then
				create {ITP_NATURAL_8} receiver.make (8)
			elseif i = 9 then
				create {ITP_NATURAL_8} receiver.make (9)
			elseif i = 10 then
				create {ITP_NATURAL_8} receiver.make (10)
			elseif i = 11 then
				create {ITP_NATURAL_8} receiver.make (20)
			elseif i = 12 then
				create {ITP_NATURAL_8} receiver.make (126)
			elseif i = 13 then
				create {ITP_NATURAL_8} receiver.make (127)
			elseif i = 14 then
				create {ITP_NATURAL_8} receiver.make (128)
			elseif i = 15 then
				create {ITP_NATURAL_8} receiver.make (129)
			elseif i = 16 then
				create {ITP_NATURAL_8} receiver.make ({NATURAL_8}.Min_value)
			elseif i = 17 then
				create {ITP_NATURAL_8} receiver.make ({NATURAL_8}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_natural_16_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_NATURAL_16} receiver.make (0)
			elseif i = 1 then
				create {ITP_NATURAL_16} receiver.make (1)
			elseif i = 2 then
				create {ITP_NATURAL_16} receiver.make (2)
			elseif i = 3 then
				create {ITP_NATURAL_16} receiver.make (3)
			elseif i = 4 then
				create {ITP_NATURAL_16} receiver.make (4)
			elseif i = 5 then
				create {ITP_NATURAL_16} receiver.make (5)
			elseif i = 6 then
				create {ITP_NATURAL_16} receiver.make (6)
			elseif i = 7 then
				create {ITP_NATURAL_16} receiver.make (7)
			elseif i = 8 then
				create {ITP_NATURAL_16} receiver.make (8)
			elseif i = 9 then
				create {ITP_NATURAL_16} receiver.make (9)
			elseif i = 10 then
				create {ITP_NATURAL_16} receiver.make (10)
			elseif i = 11 then
				create {ITP_NATURAL_16} receiver.make (20)
			elseif i = 12 then
				create {ITP_NATURAL_16} receiver.make (126)
			elseif i = 13 then
				create {ITP_NATURAL_16} receiver.make (127)
			elseif i = 14 then
				create {ITP_NATURAL_16} receiver.make (128)
			elseif i = 15 then
				create {ITP_NATURAL_16} receiver.make (129)
			elseif i = 16 then
				create {ITP_NATURAL_16} receiver.make ({NATURAL_16}.Min_value)
			elseif i = 17 then
				create {ITP_NATURAL_16} receiver.make ({NATURAL_16}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end


	choose_natural_32_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_NATURAL_32} receiver.make (0)
			elseif i = 1 then
				create {ITP_NATURAL_32} receiver.make (1)
			elseif i = 2 then
				create {ITP_NATURAL_32} receiver.make (2)
			elseif i = 3 then
				create {ITP_NATURAL_32} receiver.make (3)
			elseif i = 4 then
				create {ITP_NATURAL_32} receiver.make (4)
			elseif i = 5 then
				create {ITP_NATURAL_32} receiver.make (5)
			elseif i = 6 then
				create {ITP_NATURAL_32} receiver.make (6)
			elseif i = 7 then
				create {ITP_NATURAL_32} receiver.make (7)
			elseif i = 8 then
				create {ITP_NATURAL_32} receiver.make (8)
			elseif i = 9 then
				create {ITP_NATURAL_32} receiver.make (9)
			elseif i = 10 then
				create {ITP_NATURAL_32} receiver.make (10)
			elseif i = 11 then
				create {ITP_NATURAL_32} receiver.make (20)
			elseif i = 12 then
				create {ITP_NATURAL_32} receiver.make (126)
			elseif i = 13 then
				create {ITP_NATURAL_32} receiver.make (127)
			elseif i = 14 then
				create {ITP_NATURAL_32} receiver.make (128)
			elseif i = 15 then
				create {ITP_NATURAL_32} receiver.make (129)
			elseif i = 16 then
				create {ITP_NATURAL_32} receiver.make ({NATURAL_32}.Min_value)
			elseif i = 17 then
				create {ITP_NATURAL_32} receiver.make ({NATURAL_32}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_natural_64_receiver is
			-- Choose integer receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 18)
			if i = 0 then
				create {ITP_NATURAL_64} receiver.make (0)
			elseif i = 1 then
				create {ITP_NATURAL_64} receiver.make (1)
			elseif i = 2 then
				create {ITP_NATURAL_64} receiver.make (2)
			elseif i = 3 then
				create {ITP_NATURAL_64} receiver.make (3)
			elseif i = 4 then
				create {ITP_NATURAL_64} receiver.make (4)
			elseif i = 5 then
				create {ITP_NATURAL_64} receiver.make (5)
			elseif i = 6 then
				create {ITP_NATURAL_64} receiver.make (6)
			elseif i = 7 then
				create {ITP_NATURAL_64} receiver.make (7)
			elseif i = 8 then
				create {ITP_NATURAL_64} receiver.make (8)
			elseif i = 9 then
				create {ITP_NATURAL_64} receiver.make (9)
			elseif i = 10 then
				create {ITP_NATURAL_64} receiver.make (10)
			elseif i = 11 then
				create {ITP_NATURAL_64} receiver.make (20)
			elseif i = 12 then
				create {ITP_NATURAL_64} receiver.make (126)
			elseif i = 13 then
				create {ITP_NATURAL_64} receiver.make (127)
			elseif i = 14 then
				create {ITP_NATURAL_64} receiver.make (128)
			elseif i = 15 then
				create {ITP_NATURAL_64} receiver.make (129)
			elseif i = 16 then
				create {ITP_NATURAL_64} receiver.make ({NATURAL_64}.Min_value)
			elseif i = 17 then
				create {ITP_NATURAL_64} receiver.make ({NATURAL_64}.Max_value)
			end
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_pointer_receiver is
			-- Choose boolean receiver and set it to
			-- `receiver'.
		do
			create {ITP_POINTER} receiver.make (default_pointer)
		ensure
			receiver_not_void: receiver /= Void
		end

	choose_real_receiver is
			-- Choose boolean receiver and set it to
			-- `receiver'.
		local
			i: INTEGER
		do
			random.forth
			i := (random.item  \\ 10)
			inspect i
			when 0 then
				create {ITP_REAL} receiver.make (-1.0)
			when 1 then
				create {ITP_REAL} receiver.make (1.0)
			when 2 then
				create {ITP_REAL} receiver.make (0.0)
			when 3 then
				create {ITP_REAL} receiver.make (-2.0)
			when 4 then
				create {ITP_REAL} receiver.make (2.0)
			when 5 then
				create {ITP_REAL} receiver.make (-100)
			when 6 then
				create {ITP_REAL} receiver.make (100)
			when 7 then
				create {ITP_REAL} receiver.make (3.40282e+38)
			when 8 then
				create {ITP_REAL} receiver.make (1.17549e-38)
			when 9 then
				create {ITP_REAL} receiver.make (1.19209e-07)
			else
				check
					dead_end: False
				end
			end
		ensure
			receiver_not_void: receiver /= Void
		end

invariant

	universe_not_void: universe /= Void
	interpreter_not_void: interpreter /= Void
	type_not_void: type /= Void
	creation_procedure_belongs_to_type: creation_procedure /= Void implies (type.direct_base_class (universe).procedures.has (creation_procedure))
	is_creation_procedure: creation_procedure /= Void implies type.direct_base_class (universe).is_creation_exported_to (creation_procedure.name, universe.any_class, universe)

end
