indexing
	description: "Objects which start the ISE Eiffel compiler (out of process)"
	author: "Andreas Leitner"
	date: "$Date$"
	revision: "$Revision$"

class AUT_ISE_EIFFEL_COMPILER

inherit

	AUT_SHARED_REGISTRY
		export {NONE} all end

	EPX_CURRENT_PROCESS
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_OPERATING_SYSTEM
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make is
			-- Create new eiffel compiler that will silently surpress all output.
		do
		end

feature -- Access

	output_handler: PROCEDURE [ANY, TUPLE [STRING]]
			-- Output handler that will be invoked for any compiler output

feature -- Status setting

	set_output_handler (an_output_handler: like output_handler) is
			-- Set `output_handler' to `an_output_handler'.
		require
			an_output_handler_not_void: an_output_handler /= Void
		do
			output_handler := an_output_handler
		ensure
			output_handler_set: output_handler = an_output_handler
		end

feature -- Execution

	run (a_working_directory: STRING; an_ace_filename: STRING; a_system_name: STRING) is
			-- Run ISE Eiffel compiler in directory `a_working_directory'
			-- compiling the system described via `an_ace_file_name'.
			-- Block until compilation stops.
		require
			a_working_directory_not_void: a_working_directory /= Void
			an_ace_filename_not_void: an_ace_filename /= Void
			a_system_name_not_void: a_system_name /= Void
		do
			run_ec (a_working_directory, an_ace_filename, a_system_name)
			run_finish_freezing (a_working_directory)
		end

feature {NONE} -- Implementation

	run_ec (a_working_directory: STRING; an_ace_filename: STRING; a_system_name: STRING) is
			-- Run 'ec' in directory `a_working_directory'
			-- compiling the system named `a_system_name' described via `an_ace_filename'.
			-- Block until execution stops.
		require
			a_working_directory_not_void: a_working_directory /= Void
			an_ace_filename_not_void: an_ace_filename /= Void
			a_system_name_not_void: a_system_name /= Void
		local
			old_working_directory: STRING
			process: EPX_EXEC_PROCESS
			quoted_ace_filename: STRING
			quoted_epr_filename: STRING
		do
			if operating_system.is_windows then
				create quoted_ace_filename.make (an_ace_filename.count + 2)
				quoted_ace_filename.append_character ('"')
				quoted_ace_filename.append_string (an_ace_filename)
				quoted_ace_filename.append_character ('"')

				create quoted_epr_filename.make (a_system_name.count + 6)
				quoted_epr_filename.append_character ('"')
				quoted_epr_filename.append_string (a_system_name)
				quoted_epr_filename.append_string (".epr")
				quoted_epr_filename.append_character ('"')
			else
				quoted_ace_filename := an_ace_filename

				create quoted_epr_filename.make (a_system_name.count + 4)
				quoted_epr_filename.append_string (a_system_name)
				quoted_epr_filename.append_string (".epr")
			end
			old_working_directory := file_system.current_working_directory
			file_system.set_current_working_directory (a_working_directory)
			if file_system.file_exists (a_system_name + ".epr") and then file_system.directory_exists ("EIFGEN") then
				create process.make (eiffel_compiler_executable_name, <<"-batch", "-finalize", "-keep", "-ace", quoted_ace_filename, "-project", quoted_epr_filename>>)
			else
				create process.make (eiffel_compiler_executable_name, <<"-batch", "-finalize", "-keep", "-ace", quoted_ace_filename>>)
			end
			execute_process (process)
			file_system.set_current_working_directory (old_working_directory)
		end

	run_finish_freezing (a_working_directory: STRING) is
			-- Run 'finish_freezing' in directory `a_working_directory'
			-- Block until execution stops.
		require
			a_working_directory_not_void: a_working_directory /= Void
		local
			old_working_directory: STRING
			process: EPX_EXEC_PROCESS
		do
			old_working_directory := file_system.current_working_directory
			file_system.set_current_working_directory (file_system.nested_pathname (a_working_directory, <<"EIFGEN", "F_code">>))
			create process.make (finish_freezing_executable_name, <<"-silent">>)
			execute_process (process)
			file_system.set_current_working_directory (old_working_directory)
		end

	execute_process (a_process: EPX_EXEC_PROCESS) is
			-- Execute `a_process' and report its output on stdout and stderr
			-- via `output_handler'. Block until execution finished.
		require
			a_process_not_void: a_process /= Void
		local
			stdout_reader: AUT_NON_BLOCKING_LINE_READER
			stderr_reader: AUT_NON_BLOCKING_LINE_READER
		do
			from
				a_process.set_capture_error (True)
				a_process.set_capture_output (True)
				a_process.set_capture_input (True)
				a_process.execute
				create stdout_reader.make (a_process.fd_stdout)
				create stderr_reader.make (a_process.fd_stderr)
			until
				a_process.fd_stdout.eof and a_process.fd_stderr.eof
			loop
				if not a_process.fd_stdout.eof then
					stdout_reader.try_read_line
					if stdout_reader.has_read_line and output_handler /= Void then
						output_handler.call ([stdout_reader.last_line])
					end
				end
				if not a_process.fd_stderr.eof then
					stderr_reader.try_read_line
					if stderr_reader.has_read_line and output_handler /= Void then
						output_handler.call ([stderr_reader.last_line])
					end
				end
				millisleep (10)
			end
			a_process.wait_for (True)
		end

feature -- Constants

	eiffel_compiler_executable_name: STRING is
			-- Name of the command line ISE eiffel compiler
		require
			ise_eiffel_installed: is_ise_eiffel_installed
		once
			Result := file_system.pathname (ise_eiffel_bin_path, "ec")
			if operating_system.is_windows then
				Result.append (".exe")
			end
		ensure
			executable_name_not_void: Result /= Void
		end

	finish_freezing_executable_name: STRING is
			-- Name of the command line tool "finish_freezing"
		require
			ise_eiffel_installed: is_ise_eiffel_installed
		once
			Result := file_system.pathname (ise_eiffel_bin_path, "finish_freezing")
			if operating_system.is_windows then
				Result.append (".exe")
			end
		ensure
			executable_name_not_void: Result /= Void
		end

	is_ise_eiffel_installed: BOOLEAN is
			-- Is ISE Eiffel installed on this computer?
		once
			if operating_system.is_windows then
				check
					registry_available: registry.is_available
				end
				Result := latest_ise_eiffel_key /= Void
			else
				Result := ise_eiffel_variable /= Void and ise_platform_variable /= Void
			end
		end

	ise_eiffel_path: STRING is
			-- Path to ISE Eiffel directory
		require
			ise_eiffel_installed: is_ise_eiffel_installed
		once
			if operating_system.is_windows then
				check
					registry_available: registry.is_available
				end
				Result := registry.string_value (latest_ise_eiffel_key + "\ec\ISE_EIFFEL")
			else
				Result := ise_eiffel_variable
			end
		ensure
			path_not_void: Result /= Void
		end

	ise_eiffel_bin_path: STRING is
			-- Path to ISE Eiffel directory
		require
			ise_eiffel_installed: is_ise_eiffel_installed
		once
			Result := file_system.nested_pathname (ise_eiffel_path, <<"studio", "spec", ise_platform_name, "bin">>)
		ensure
			path_not_void: Result /= Void
			path_not_empty: Result.count > 0
		end

	ise_platform_name: STRING is
			-- Platform name of current computer
		require
			ise_eiffel_installed: is_ise_eiffel_installed
		do
			if operating_system.is_windows then
				Result := "windows"
			else
				Result := ise_platform_variable
			end
		ensure
			name_not_void: Result /= Void
			name_not_empty: Result.count > 0
		end

	ise_eiffel_variable_name: STRING is
			-- Name of ISE Eiffel environment variable
		once
			Result := "ISE_EIFFEL"
		ensure
			name_not_void: Result /= Void
			name_correct: Result.is_equal ("ISE_EIFFEL")
		end

	ise_eiffel_variable: STRING is
			-- Content of environment variable ${ISE_EIFFEL};
			-- `Void' if not set.
		once
			Result := execution_environment.variable_value (ise_eiffel_variable_name)
		end

	ise_platform_variable: STRING is
			-- Content of environment variable ${ISE_PLATFORM};
			-- `Void' if not set.
		once
			Result := execution_environment.variable_value ("ISE_PLATFORM")
		end

feature {NONE} -- Implementation

	ise_eiffel_common_root_key: STRING is "HKEY_LOCAL_MACHINE\SOFTWARE\ISE"
			-- Key where ISE Eiffel installation will register themselves

	latest_ise_eiffel_key: STRING is
			-- Registry key to the latest installed version of ISE Eiffel
			-- or `Void' if no version installed at all
		require
			registry_available: registry.is_available
		local
			list: DS_LINEAR [STRING]
			cs: DS_LINEAR_CURSOR [STRING]
			matcher: RX_PCRE_MATCHER
			version_string: STRING
			version_number: INTEGER
			max_version_number: INTEGER
		do
			create matcher.make
			matcher.compile ("Eiffel([0-9][0-9])")
			check
				compiled_sucessfully: matcher.is_compiled
			end
			from
				list := registry.subkeys (ise_eiffel_common_root_key)
				cs := list.new_cursor
				cs.start
			until
				cs.off
			loop
				matcher.match (cs.item)
				if matcher.has_matched then
					check
						version_number_match: matcher.is_captured_substring_defined (1)
					end
					version_string := matcher.captured_substring (1)
					check
						valid_number: version_string.is_integer
					end
					version_number := version_string.to_integer
					if version_number > max_version_number then
						max_version_number := version_number
						Result := cs.item
					end
				end
				cs.forth
			end
			if Result /= Void then
				Result.prepend_character ('\')
				Result.prepend_string (ise_eiffel_common_root_key)
			end
		ensure
			key_not_empty: Result /= Void implies Result.count > 0
		end

end
